@@ -6,7 +6,7 @@ import gevent
 # demo app
 import os
 import random
-def handle(environ, start_response, ws):
+def handle(ws):
     """  This is the websocket handler function.  Note that we
     can dispatch based on path in here, too."""
     if ws.path == '/echo':
@@ -22,6 +22,19 @@ def handle(environ, start_response, ws):
             #print "0 %s %s\n" % (i, random.random())
             gevent.sleep(0.1)
 
-server = pywsgi.WSGIServer(('0.0.0.0', 9999), handle,
+
+def app(environ, start_response):
+    if environ['PATH_INFO'] == '/test':
+        start_response("200 OK", [('Content-Type', 'text/plain')])
+        return ["blaat"]
+    elif environ['PATH_INFO'] == "/data":
+        handle(environ['wsgi.websocket'])
+    else:
+        start_response("404 Not Found", [])
+        return []
+
+
+
+server = pywsgi.WSGIServer(('0.0.0.0', 8000), app,
         handler_class=WebSocketHandler)
 server.serve_forever()
