@@ -1,15 +1,28 @@
 import re
 import struct
-import time
-import traceback
-import sys
 from hashlib import md5
+
 from gevent.pywsgi import WSGIHandler
 from geventwebsocket import WebSocket
 
+
+class HandShakeError(ValueError):
+    """ Hand shake challenge can't be parsed """
+    pass
+
+
 class WebSocketHandler(WSGIHandler):
+    """ Automatically upgrades the connection to websockets. """
     def __init__(self, *args, **kwargs):
         self.websocket_connection = False
+        self.allowed_paths = []
+
+        for expression in kwargs.pop('allowed_paths', []):
+            if isinstance(expression, basestring):
+                self.allowed_paths.append(re.compile(expression))
+            else:
+                self.allowed_paths.append(expression)
+
         super(WebSocketHandler, self).__init__(*args, **kwargs)
 
     def handle_one_response(self, call_wsgi_app=True):
@@ -17,12 +30,13 @@ class WebSocketHandler(WSGIHandler):
         # we will proceed with the default PyWSGI functionality.
         if self.environ.get("HTTP_CONNECTION") != "Upgrade" or \
            self.environ.get("HTTP_UPGRADE") != "WebSocket" or \
-           not self.environ.get("HTTP_ORIGIN"):
+           not self.environ.get("HTTP_ORIGIN") or \
+           not self.accept_upgrade():
             return super(WebSocketHandler, self).handle_one_response()
         else:
             self.websocket_connection = True
 
-        self.websocket = WebSocket(self.rfile, self.wfile, self.socket, self.environ)
+        self.websocket = WebSocket(self.socket, self.rfile, self.environ)
         self.environ['wsgi.websocket'] = self.websocket
 
         # Detect the Websocket protocol
@@ -51,7 +65,7 @@ class WebSocketHandler(WSGIHandler):
             ]
 
             self.start_response("101 Web Socket Protocol Handshake", headers)
-            self.write([challenge])
+            self.write(challenge)
         else:
             raise Exception("Version not supported")
 
@@ -60,9 +74,24 @@ class WebSocketHandler(WSGIHandler):
         else:
             return
 
+    def accept_upgrade(self):
+        """
+        Returns True if request is allowed to be upgraded.
+        If self.allowed_paths is non-empty, self.environ['PATH_INFO'] will
+        be matched against each of the regular expressions.
+        """
+
+        if self.allowed_paths:
+            path_info = self.environ.get('PATH_INFO', '')
+
+            for regexps in self.allowed_paths:
+                return regexps.match(path_info)
+        else:
+            return True
+
     def write(self, data):
         if self.websocket_connection:
-            self.wfile.writelines(data)
+            self.socket.sendall(data)
         else:
             super(WebSocketHandler, self).write(data)
 
@@ -77,7 +106,8 @@ class WebSocketHandler(WSGIHandler):
                 towrite.append("%s: %s\r\n" % header)
 
             towrite.append("\r\n")
-            self.wfile.writelines(towrite)
+            msg = ''.join(towrite)
+            self.socket.sendall(msg)
             self.headers_sent = True
         else:
             super(WebSocketHandler, self).start_response(status, headers, exc_info)
@@ -100,7 +130,7 @@ class WebSocketHandler(WSGIHandler):
             message = "Client using old/invalid protocol implementation"
             headers = [("Content-Length", str(len(message))),]
             self.start_response("400 Bad Request", headers)
-            self.write([message])
+            self.write(message)
             self.close_connection = True
             return
 
